"use strict";

define([
    "jquery",
    "loglevel"
],
function ($, loglevel) {
    var chartWidth,
        chartHeight,
        dcvConsoleCanvas,
        verticalPadding = 60,
        horizontalPadding = 20,
        highestY = 0,
        ctx;

    function drawBackgroundRectangle () {
        var x,
            horizontalOffset;

        // frame
        ctx.moveTo(horizontalPadding / 2, 0);
        ctx.lineTo(chartWidth + horizontalPadding / 2, 0);
        ctx.stroke();

        ctx.moveTo(chartWidth + horizontalPadding / 2, 0);
        ctx.lineTo(chartWidth + horizontalPadding / 2, chartHeight);
        ctx.stroke();

        ctx.moveTo(chartWidth + horizontalPadding / 2, chartHeight);
        ctx.lineTo(horizontalPadding / 2, chartHeight);
        ctx.stroke();

        ctx.moveTo(horizontalPadding / 2, chartHeight);
        ctx.lineTo(horizontalPadding / 2, 0);
        ctx.stroke();

        // horizontal tick lines

        ctx.moveTo(horizontalPadding / 2, chartHeight / 2);
        ctx.lineTo(chartWidth + horizontalPadding / 2, chartHeight / 2);
        ctx.stroke();

        // vertical tick lines

        horizontalOffset = chartWidth / 6;

        for (x = horizontalOffset; x < chartWidth; x += horizontalOffset) {
            ctx.moveTo(x + horizontalPadding / 2, 0);
            ctx.lineTo(x + horizontalPadding / 2, chartHeight);
            ctx.stroke();
        }
    }

    function drawBackground () {
        ctx.beginPath();

        ctx.lineWidth = 1;
        ctx.strokeStyle = "#000";
        ctx.setLineDash([2, 2]);

        drawBackgroundRectangle();
    }

    function drawAverage (average) {
        var y = chartHeight - average;

        ctx.beginPath();

        ctx.lineWidth = 1;
        ctx.strokeStyle = "#f00";
        ctx.setLineDash([3, 1]);

        ctx.moveTo(horizontalPadding / 2, y);
        ctx.lineTo(chartWidth + horizontalPadding / 2, y);

        ctx.stroke();
    }

    function drawBars (data, average) {
        var i,
            numClearBars,
            offset,
            x = 0,
            y,
            percentage;

        if (data && data.length > 0) {
            for (i = 0; i < data.length; i++) {
                if (data[i].y > highestY) {
                    highestY = data[i].y;
                }
            }

            percentage = (average * 0.8) / highestY;
            drawAverage(percentage * chartHeight);
            offset = chartWidth / 60;
            numClearBars = 59 - data.length;

            for (i = 0; i < numClearBars; i++) {
                x += offset;
            }

            for (i = 0; i < data.length; i++) {
                x += offset;
                percentage = (data[i].y * 0.8) / highestY;
                y = percentage * chartHeight;

                ctx.beginPath();
                ctx.setLineDash([]);
                ctx.lineWidth = 4;
                ctx.strokeStyle = "#249ED5";
                ctx.moveTo(x + horizontalPadding / 2, chartHeight);
                ctx.lineTo(x + horizontalPadding / 2, chartHeight - y);
                ctx.stroke();
            }
        }
    }

    function format (bps) {
        if (bps < 1000) {
            return bps.toFixed(1) + " bps";
        } else if (bps < 1000 * 1000) {
            return (bps / 1000).toFixed(1) + " kbps";
        }
        return (bps / (1000 * 1000)).toFixed(1) + " Mbps";
    }

    function drawLabels () {
        var i,
            offset = chartWidth / 6,
            yLabelOffset = chartWidth + 35,
            horizontalLabels;

        ctx.beginPath();
        ctx.font = "9px arial";
        ctx.textBaseline = "middle";
        ctx.textAlign = "center";
        ctx.strokeStyle = "#000";

        horizontalLabels = ["60 seconds", "50", "40", "30", "20", "10", "0"];

        for (i = 0; i < horizontalLabels.length; i++) {
            if (i === 0) {
                ctx.fillText(horizontalLabels[i], 30, chartHeight + 10);
            } else {
                ctx.fillText(horizontalLabels[i], i * offset + 10, chartHeight + 10);
            }
        }

        ctx.fillText("0 bps", yLabelOffset, chartHeight);
        ctx.fillText(format(highestY / 2), yLabelOffset, chartHeight / 2);
        ctx.fillText(format(highestY), yLabelOffset, 4);
    }

    return {
        drawChart: function (data, average) {
            dcvConsoleCanvas = $("#dcv-console-canvas")[0];
            chartWidth = dcvConsoleCanvas.width - verticalPadding;
            chartHeight = dcvConsoleCanvas.height - horizontalPadding;
            ctx = dcvConsoleCanvas.getContext("2d");

            ctx.clearRect(0, 0, dcvConsoleCanvas.width, dcvConsoleCanvas.height);

            drawBackground();
            drawLabels();
            drawBars(data);
        }
    };
});

/* ex:set ts=4 et: */
