"use strict";

define([
    "jquery",
    "loglevel",
    "jqueryui",
    "./icons",
    "./bandwidthchart",
    "./settings",
    "./utils",
    "bootstrap" // does not return an object, must be after the other modules
],
function ($, loglevel, jqueryui, icons, bandwidthChart, settings, utils) {

    var log,
        currentTrafficLabel,
        averageTrafficLabel,
        peakTrafficLabel,
        latencyLabel,
        fpsLabel,
        visible,
        chartData,
        minQuality;

    function updateStats (connection) {
        var stats = connection.getStats(),
            lastPoint,
            tmp,
            i,
            average = 0;

        if (!chartData) {
            chartData = [];
        }

        if (chartData.length > 59) {
            tmp = [];

            for (i = 1; i < chartData.length; i++) {
                tmp.push(chartData[i]);
            }

            chartData = tmp;
        }

        if (chartData.length === 0) {
            chartData.push({
                x: 0,
                y: stats.traffic,
                q: minQuality
            });
        } else {
            lastPoint = chartData[chartData.length - 1];
            chartData.push({
                x: lastPoint.x + 1,
                y: stats.traffic,
                q: minQuality
            });
        }

        /* Calculate average */
        for (i = 0; i < chartData.length; i++) {
            average += chartData[i].y;
        }

        average /= chartData.length;

        /* Update labels */
        currentTrafficLabel.text(utils.formatTraffic(stats.traffic));
        averageTrafficLabel.text(utils.formatTraffic(average));
        peakTrafficLabel.text(utils.formatTraffic(stats.peakTraffic));
        latencyLabel.text(stats.latency > 999 ? "> 1 sec" : Math.round(stats.latency) + " ms");
        fpsLabel.text(stats.fps + " fps");

        /* Update chart */
        bandwidthChart.drawChart(chartData, average);

        if (visible) {
            setTimeout(updateStats, 1000, connection);
        }
    }

    /* In some cases the browser's window is smaller than the container (ie server
     * resolution is larger than window and we are keeping the server resolution).
     * In this case we must avoid going out the window since it will break again
     * the input system.
     */
    function keepItOnVisible () {
        var newTop = $("#dcv-console").offset().top,
            newLeft = $("#dcv-console").offset().left;

        if (visible) {
            if (($(window).width() - $("#dcv-console").width() - $("#dcv-console").offset().left) < 0) {
                newLeft = Math.max($("#dcv-display-frame").offset().left, $(window).width() - $("#dcv-console").width());
            }
            if (($(window).height() - $("#dcv-console").height() - $("#dcv-console").offset().top) < 0) {
                newTop = Math.max($("#dcv-display-frame").offset().top, $(window).height() - $("#dcv-console").height());
            }
            $("#dcv-console").offset({top: newTop, left: newLeft});
        }
    }

    return {
        setup: function (consoleDialog, connection, defaultMinQuality) {
            var minQualitySmooth = 20,
                maxQualitySmooth = 90,
                minQualitySharp = 70,
                maxQualitySharp = 90;

            log = loglevel.getLogger("console");
            currentTrafficLabel = $("#dcv-console-current-traffic-label", consoleDialog);
            averageTrafficLabel = $("#dcv-console-average-traffic-label", consoleDialog);
            peakTrafficLabel = $("#dcv-console-peak-traffic-label", consoleDialog);
            latencyLabel = $("#dcv-console-latency-label", consoleDialog);
            fpsLabel = $("#dcv-console-fps-label", consoleDialog);

            $("#dcv-streaming-mode").click(function () {
                consoleDialog.css({
                    "display": "block",
                    "position": "absolute",
                    "top": "10%",
                    "left": "30%",
                    "z-index": "4"
                });
                visible = true;
                keepItOnVisible();
                chartData = [];
                updateStats(connection);
            });

            $("#dcv-console-close").click(function () {
                consoleDialog.css({
                    display: "none"
                });
                visible = false;
            });

            $("body").keydown(function (e) {
                if (e.keyCode === 27) {
                    consoleDialog.css({
                        display: "none"
                    });
                    visible = false;
                }
            });

            consoleDialog.draggable({
                containment: "#dcv-display-frame",
                handle: ".modal-header",
                stop: keepItOnVisible,
                scroll: false
            });

            window.addEventListener("resize", keepItOnVisible);

            $("#display-metrics").on("show.bs.collapse", function () {
                $("#dcv-display-metrics").html(icons["arrow-down"] + "Hide Streaming Metrics");
            }).on("shown.bs.collapse", function () {
                keepItOnVisible();
            }).on("hide.bs.collapse", function () {
                $("#dcv-display-metrics").html(icons["arrow-right"] + "Display Streaming Metrics");
            });

            $("input[type=radio][name=display-quality]").change(function (e) {

                /* eslint-disable no-invalid-this */
                if (this.value === "smooth") {
                    minQuality = minQualitySmooth;
                    connection.setDisplayQuality(minQualitySmooth, maxQualitySmooth);
                    log.debug("Switching to smooth display mode");
                } else if (this.value === "sharp") {
                    minQuality = minQualitySharp;
                    connection.setDisplayQuality(minQualitySharp, maxQualitySharp);
                    log.debug("Switching to sharp display mode");
                }

                /* eslint-enable no-invalid-this */
            });

            bandwidthChart.drawChart([], 0);
        }
    };
});

/* ex:set ts=4 et: */
