"use strict";

define([
    "loglevel",
    "./utils"
],
function (loglevel, utils) {
    var log,
        conn,
        callbacks = {},
        pendingUpdates = [];

    function queryFeature (feat) {
        conn.queryFeature(feat).then(function (status) {
            callbacks[feat](status);
        }).catch(function (error) {
            log.info("[UI][Feature handler] Query for " + feat + " denied with message: " + error.message);
        });
    }

    function featuresUpdated (list) {
        list.forEach(function (feat) {
            if (Object.keys(callbacks).indexOf(feat) !== -1) {
                queryFeature(feat);
            } else {
                pendingUpdates.push(feat);
                log.info("[UI][Feature handler] No callback for feature " + feat + " adding to pending.");
            }
        });
    }

    return {
        setup: function (connection) {
            log = loglevel.getLogger("featurehandler");
            conn = connection;
        },

        featuresUpdate: function (list) {
            if (list.length > 0) {
                featuresUpdated(list);
            }
        },

        addFeatureListener: function (name, func) {
            if (name && utils.isFunction(func)) {
                callbacks[name] = func;
                log.info("[UI] Added feature listener for: " + name);
                log.debug("[UI] Callbacks " + Object.keys(callbacks));
                log.debug("[UI][Feature] Pending updates " + pendingUpdates);
                if (pendingUpdates.indexOf(name) !== -1) {
                    queryFeature(name);
                    pendingUpdates.splice(pendingUpdates.indexOf(name), 1);
                }
            } else {
                log.warning("[UI] Wrong parameters for addFeatureListener");
            }
        }
    };
});

/* ex:set ts=4 et: */
