"use strict";

define([
    "jquery",
    "loglevel",
    "./notificationmanager"
],
function ($, loglevel, notificationManager) {
    var log,
        previousNotification = null,
        licenseNotifications;

    function notifyLicenseStatus (licenseNotification) {
        var leftDaysMin = 365,
            leftDaysMinProduct = "",
            notificationLevel,
            notificationText = "";

        licenseNotification.licenses.forEach(function (licenseItem) {
            if (licenseItem.isDemo) {
                log.info("License status: product '" + licenseItem.product + "' is using a demo license (" + licenseItem.leftDays + " days left)");
            }

            if (licenseItem.leftDays > 0 && licenseItem.leftDays < leftDaysMin) {
                leftDaysMin = licenseItem.leftDays;
                leftDaysMinProduct = licenseItem.product;
            }
        });

        if (licenseNotification.numUnlicensed === 0) {
            log.debug("License status: " + licenseNotification.licenses.length + " products licensed");

            if (licenseNotification.licensingMode === "demo") {
                notificationLevel = notificationManager.level.INFO;
                notificationText += "DCV is running with a demo license.";
            }

            if (leftDaysMin < 7) {
                if (notificationText !== "") {
                    notificationText += "<br>";
                }

                log.info("License status: product '" + leftDaysMinProduct + "' will expire in " + leftDaysMin + " days");
                notificationLevel = notificationManager.level.INFO;
                notificationText += "License of '" + leftDaysMinProduct + "' will expire in " + leftDaysMin + " days.";
            }
        } else {
            log.warn("License error (mode: " + licenseNotification.licensingMode + "), " + licenseNotification.numUnlicensed + "/" + licenseNotification.licenses.length +
                " products unlicensed: " + JSON.stringify(licenseNotification.licenses));

            if (licenseNotification.licensingMode === "ec2") {
                notificationLevel = notificationManager.level.WARNING;
                notificationText = "No license available.<br>Please, check your EC2 configuration";

                if (licenseNotification.documentationUrl !== null && licenseNotification.documentationUrl !== "") {
                    notificationText += "(see <a target=\"_blank\" href=\"" + licenseNotification.documentationUrl + "\">documentation</a>).";
                } else {
                    notificationText += ".";
                }
            } else {
                notificationLevel = notificationManager.level.ERROR;
                notificationText = "No license available.<br>Please, contact the administrator.";
            }
        }

        if (notificationText === "") {
            if (previousNotification) {
                previousNotification.remove();
                previousNotification = null;
            }
            return;
        }

        previousNotification = licenseNotifications.addNotification({
            level: notificationLevel,
            text: notificationText
        }, previousNotification);

    }

    return {
        setup: function () {
            log = loglevel.getLogger("license");

            licenseNotifications = notificationManager.getCategoryHandler("license");
        },

        handleNotification: function (notification) {
            notifyLicenseStatus(notification);
        }
    };
});

/* ex:set ts=4 et: */
