"use strict";

define([
    "jquery",
    "loglevel",
    "./icons",
    "./settings",
    "./utils"
],
function ($, loglevel, icons, settings, utils) {
    var log,
        button,
        container,
        notificationLevel = {ERROR: 0, WARNING: 1, INFO: 2},
        list,
        notificationsNum = 0,
        visible = false,
        notificationElement,
        categoryElement,
        categoryTitleElement,
        numNotificationsPerLevel,
        browserSupportDownload;

    function buttonClicked () {
        if (notificationsNum > 0) {
            if (visible) {
                container.hide();
            } else {
                container.show();
            }
            visible = !visible;
        }
    }

    function clearContainer () {
        container.empty();
        numNotificationsPerLevel = [0, 0, 0];
    }

    function updateContainer () {
        var i,
            len,
            categories,
            levelKeys;

        clearContainer();

        if (notificationsNum <= 0) {
            notificationsNum = 0;
            container.hide();
            button.hide();
        } else {
            categories = Object.getOwnPropertyNames(list.elem);
            for (i = 0; i < list.size; ++i) {
                len = list.elem[categories[i]].notifications.length;

                if (len > 0) {
                    levelKeys = Object.keys(notificationLevel);

                    /* eslint-disable no-loop-func */

                    levelKeys.forEach(function (key) {
                        showAll(list.elem[categories[i]], notificationLevel[key]);
                    });

                    /* eslint-enable no-loop-func */

                }
            }
        }

        updateNotificationNum();
    }

    function updateNotificationNum () {
        var text = "&nbsp" + notificationsNum + "&nbsp";

        if (notificationsNum > 0) {
            $("#dcv-notifications-num").html(text);
            if (numNotificationsPerLevel[notificationLevel.ERROR] > 0) {
                $("#dcv-notifications-num").removeClass().addClass("dcv-notifications-num-error");
            } else if (numNotificationsPerLevel[notificationLevel.WARNING] > 0) {
                $("#dcv-notifications-num").removeClass().addClass("dcv-notifications-num-warning");
            } else {
                $("#dcv-notifications-num").removeClass().addClass("dcv-notifications-num-info");
            }
            $("#dcv-notifications-num").show();
        } else {
            $("#dcv-notifications-num").hide();
        }
    }

    function showAll (elem, level) {
        var newCategory,
            newCategoryTitle,
            notificationElems = elem.getNotificationsOfLevel(level),
            idCategory = "",
            idCategoryTitle = "",
            title = "",
            i;

        if (notificationElems !== null && notificationElems.length < 1) {
            return;
        }

        newCategory = categoryElement.clone(true);
        newCategoryTitle = categoryTitleElement.clone(true);
        switch (level) {
        case notificationLevel.ERROR:
            idCategory = "dcv-category-" + elem.name + "-error";
            newCategory.prop("id", idCategory);
            idCategoryTitle = "dcv-category-title-" + elem.name + "-error";
            newCategoryTitle.prop("id", idCategoryTitle);
            newCategoryTitle.addClass("dcv-notifications-category-error");
            title = icons["error-notification"] + "&ensp; " + elem.name.toUpperCase();
            newCategoryTitle.html(title);
            numNotificationsPerLevel[level]++;
            break;
        case notificationLevel.WARNING:
            idCategory = "dcv-category-" + elem.name + "-warning";
            newCategory.prop("id", idCategory);
            idCategoryTitle = "dcv-category-title-" + elem.name + "-warning";
            newCategoryTitle.prop("id", idCategoryTitle);
            newCategoryTitle.addClass("dcv-notifications-category-warning");
            title = icons["alert-notification"] + "&ensp; " + elem.name.toUpperCase();
            newCategoryTitle.html(title);
            numNotificationsPerLevel[level]++;
            break;
        case notificationLevel.INFO:
            idCategory = "dcv-category-" + elem.name + "-info";
            newCategory.prop("id", idCategory);
            idCategoryTitle = "dcv-category-title-" + elem.name + "-info";
            newCategoryTitle.prop("id", idCategoryTitle);
            newCategoryTitle.addClass("dcv-notifications-category-info");
            title = icons["info-notification"] + "&ensp; " + elem.name.toUpperCase();
            newCategoryTitle.html(title);
            numNotificationsPerLevel[level]++;
            break;
        default:
            log.warn("[Notifications] Wrong level value for notification");
            return;
        }
        newCategory.append(newCategoryTitle);

        for (i = 0; i < notificationElems.length; ++i) {
            newCategory.append(displayNotification(notificationElems[i], idCategory, i));
        }

        container.append(newCategory);
    }

    /* eslint-disable no-loop-func*/

    function displayNotification (elem, idCategory, idx) {
        var newNotification,
            idString = "",
            content,
            code,
            i,
            actionId,
            actionElem,
            filename;

        idString = idCategory + "-not-" + idx;
        newNotification = notificationElement.clone();
        newNotification.prop("id", idString);
        elem.id = idString;
        content = newNotification.find(".dcv-notification-content");

        if (isNotUndefinedOrNull(elem.text)) {
            content.find(".dcv-notification-text").append(elem.text + "<br />");
        }

        for (i = 0; i < elem.actions.length; ++i) {
            if (elem.actions[i].hasOwnProperty("text") &&
                elem.actions[i].hasOwnProperty("link") &&
                isNotUndefinedOrNull(elem.actions[i].link)) {
                actionId = "dcv-notification-" + idx + "-" + i;
                code = "<a id=\"" + actionId + "\" href=\"" + elem.actions[i].link + "\" target=\"_blank\">" + elem.actions[i].text + "</a><br />";
                content.find(".dcv-notification-text").append(code);
                if (elem.actions[i].remove) {
                    actionId = "#" + actionId;
                    actionElem = content.find(actionId);
                    actionElem.on("click", function () {
                        arguments[0].remove();
                    }.bind(null, elem));
                }
            }
            if (elem.actions[i].hasOwnProperty("text") &&
                elem.actions[i].hasOwnProperty("download") &&
                isNotUndefinedOrNull(elem.actions[i].download)) {
                filename = elem.actions[i].download.split("/");
                actionId = "dcv-notification-" + idx + "-" + i;
                if (browserSupportDownload) { // checks if the HTML5 download attribute is supported
                    code = "<a id=\"" + actionId + "\" href=\"" + elem.actions[i].download + "\" target=\"_blank\" download=\"" + filename[filename.length - 1] + "\">" + elem.actions[i].text + "</a><br />";
                } else {
                    code = "<a id=\"" + actionId + "\" href=\"" + elem.actions[i].download + "\" target=\"_blank\">" + elem.actions[i].text + " (Use right-click menu to save it)</a><br />";
                }
                content.find(".dcv-notification-text").append(code);
                if (elem.actions[i].remove) {
                    actionId = "#" + actionId;
                    actionElem = content.find(actionId);
                    actionElem.on("click", function () {
                        arguments[0].remove();
                    }.bind(null, elem));
                }
            }
            if (elem.actions[i].hasOwnProperty("text") &&
                elem.actions[i].hasOwnProperty("callback") &&
                utils.isFunction(elem.actions[i].callback)) {
                actionId = "dcv-notification-" + idx + "-" + i;
                code = "<a id=\"" + actionId + "\" href=\"#\">" + elem.actions[i].text + "</a><br />";
                content.find(".dcv-notification-text").append(code);
                actionId = "#" + actionId;
                actionElem = content.find(actionId);
                actionElem.on("click", function () {
                    arguments[0].callback();
                }.bind(null, elem.actions[i]));
                if (elem.actions[i].remove) {
                    actionElem.on("click", function () {
                        arguments[0].remove();
                    }.bind(null, elem));
                }
            }
            if (elem.actions[i].hasOwnProperty("dismissable") &&
                elem.actions[i].dismissable) {
                actionId = "dcv-notification-" + idx + "-" + i;
                code = "<a id=\"" + actionId + "\" href=\"#\">Dismiss notification</a><br />";
                content.find(".dcv-notification-text").append(code);
                actionId = "#" + actionId;
                actionElem = content.find(actionId);
                actionElem.on("click", function () {
                    arguments[0].remove();
                }.bind(null, elem));
            }
        }

        if (!isNotUndefinedOrNull(elem.progress)) {
            content.find(".dcv-notification-progress-bg").remove();
            content.find(".dcv-notification-progress-value").remove();
        } else {
            content.find(".dcv-notification-text").append("<br />");
            content.find(".dcv-notification-progress-value").html(elem.progress + "%");
            content.find(".dcv-notification-progress-bar").css("width", elem.progress + "%");
        }

        return newNotification;
    }

    /* eslint-enable no-loop-func*/

    function isNotUndefinedOrNull (val) {
        return !((val === null) || (typeof val === "undefined"));
    }

    /* NotificationsList class. Contains the list of notifications divided
     * per category.
     */

    function NotificationsList () {
        this.elem = {};
        this.size = 0;

        this.addCategory = function (key) {
            var newCategory;

            if (!this.elem.hasOwnProperty(key)) {
                newCategory = new NotificationCategory(key);
                this.size++;
                this.elem[key] = newCategory;
                log.debug("[Notifications] Added new category " + key);
            }

            return this.elem[key];
        };

        this.removeCategory = function (key) {
            if (this.elem.hasOwnProperty(key)) {
                delete this.elem[key];
                this.size--;
            }
        };
    }

    /* NotificationCategory contains all the notifications for a category */

    function NotificationCategory (key) {
        this.name = key;
        this.notifications = [];

        function findNotification (notifArray, notification) {
            var i;

            if (notification.id !== "" && !notification.toRemove) {
                for (i = 0; i < notifArray.length; ++i) {
                    if (notification.id === notifArray[i].id) {
                        return i;
                    }
                }
            }

            return -1;
        }

        this.addNotification = function (data, notificationToReplace) {
            var notification = new Notification(data),
                pos;

            button.show();
            if (isNotUndefinedOrNull(notificationToReplace)) {
                pos = findNotification(this.notifications, notificationToReplace);
                if (pos > -1) {
                    this.notifications[pos] = notification;
                }
            } else {
                notificationsNum++;
                this.notifications.push(notification);
            }
            updateContainer();

            return notification;
        };

        this.getNotificationsOfLevel = function (level) {
            var tempArray = [],
                i,
                toRemove = [];

            for (i = 0; i < this.notifications.length; ++i) {
                if (this.notifications[i].level === level) {
                    if (this.notifications[i].toRemove) {
                        toRemove.push(i);
                    } else {
                        tempArray.push(this.notifications[i]);
                    }
                }
            }

            for (i = toRemove.length; i > 0; --i) {
                this.notifications.splice(toRemove[i - 1], 1);
            }

            return tempArray;
        };
    }

    function Notification (data) {
        this.id = "";
        this.level = isNotUndefinedOrNull(data.level) ? data.level : null;
        this.text = isNotUndefinedOrNull(data.text) ? data.text : null;
        this.progress = isNotUndefinedOrNull(data.progress) ? data.progress : null;
        this.actions = isNotUndefinedOrNull(data.actions) ? data.actions : [];
        this.toRemove = false;

        this.updateProgress = function (v) {
            var idString = "#" + this.id;

            this.progress = v;
            $(idString).find(".dcv-notification-progress-value").html(this.progress + "%");
            $(idString).find(".dcv-notification-progress-bar").css("width", this.progress + "%");
        };

        this.remove = function () {
            this.toRemove = true;
            notificationsNum--;
            updateContainer();
        };
    }

    return {
        setup: function (uiButton, uiContainer) {
            log = loglevel.getLogger("notificationmanager");

            button = uiButton; // we assume that is a jquery object
            container = uiContainer;

            button.hide();
            container.hide();
            $("#dcv-notifications-num").hide();

            button.on("click", buttonClicked);

            list = new NotificationsList();
            list.addCategory("error");

            notificationElement = $("#dcv-notification-template").clone(true);
            $("#dcv-notification-template").remove();
            categoryTitleElement = $("#dcv-notification-category-title-template").clone(true);
            $("#dcv-notification-category-title-template").remove();
            categoryElement = $("#dcv-notification-category-template").clone(true);
            $("#dcv-notification-category-template").remove();

            browserSupportDownload = document.createElement("a").download;
            if (typeof browserSupportDownload !== "undefined" && browserSupportDownload !== false) {
                browserSupportDownload = true;
            } else {
                browserSupportDownload = false;
            }
        },

        level: notificationLevel,

        getCategoryHandler: function (name) {
            return list.addCategory(name);
        }
    };
});

/* ex:set ts=4 et: */
