"use strict";

define([
    "bowser",
    "./featurehandler"
],
function (bowser, featurehandler) {
    var connection,
        button,
        softKeyboardInput,
        softKeyboardInputParent = "soft-kbd-div",
        softKeyboardVisible;

    function toggleSoftKeyboard () {
        if (softKeyboardVisible) {
            hideSoftKeyboard();
        } else {
            showSoftKeyboard();
        }
    }

    function showSoftKeyboard () {
        var elem = null;

        document.getElementById(softKeyboardInputParent).appendChild(softKeyboardInput);
        elem = document.getElementById("dcv-softkbd-hid-input");

        if (elem && !softKeyboardVisible) {
            elem.focus();
            if (bowser.ios) {
                elem.click();
            }
            softKeyboardVisible = true;
            elem.addEventListener("focusout", keepSoftKeyboard);
            elem.addEventListener("keydown", connection.sendKeyboardEvent);
            elem.addEventListener("keyup", connection.sendKeyboardEvent);
            elem.addEventListener("keypress", connection.sendKeyboardEvent);
        }
    }

    function hideSoftKeyboard () {
        var elem = null;

        elem = document.getElementById("dcv-softkbd-hid-input");

        if (elem && softKeyboardVisible) {
            elem.removeEventListener("focusout", keepSoftKeyboard);
            elem.removeEventListener("keydown", connection.sendKeyboardEvent);
            elem.removeEventListener("keyup", connection.sendKeyboardEvent);
            elem.removeEventListener("keypress", connection.sendKeyboardEvent);
            softKeyboardVisible = false;
            elem.blur();

            // display.focusDisplayDiv();
            elem = document.getElementById("dcv-softkbd-hid-input");
            document.getElementById(softKeyboardInputParent).removeChild(elem);
        }
    }

    function keepSoftKeyboard () {
        var elem = null;

        elem = document.getElementById("dcv-softkbd-hid-input");

        if (elem) {
            elem.focus();
            if (bowser.ios) {
                elem.click();
            }
            softKeyboardVisible = true;

            // The following is because sometimes the page is scrolled on iPad/iPhone
            if (bowser.ios) {
                window.scrollTo(0, 0);
                document.body.scrollTop = 0;
            }
        }
    }

    function createSoftKeyboardInput () {
        var elem;

        elem = document.createElement("input");
        elem.setAttribute("id", "dcv-softkbd-hid-input");
        elem.setAttribute("type", "text");
        elem.style.background = "transparent";
        elem.style.border = "none";
        elem.style.fontSize = "1pt";
        elem.style.height = "1px";
        elem.style.width = "1px";
        elem.value = "";
        softKeyboardInput = elem;
    }

    function removeSoftKeyboardInput () {
        var elem;

        if (softKeyboardInput !== null) {
            elem = document.getElementById("dcv-softkbd-hid-input");
            document.getElementById(softKeyboardInputParent).removeChild(elem);
            softKeyboardInput = null;
            softKeyboardInputParent = null;
        }
    }

    function enableSoftKeyboard (status) {
        if (status.enabled) {
            button.show(0);
            softKeyboardVisible = false;
            createSoftKeyboardInput();
            button.on("click", toggleSoftKeyboard);
        } else {
            if (softKeyboardVisible) {
                hideSoftKeyboard();
            }

            removeSoftKeyboardInput();
            button.off("click", toggleSoftKeyboard);
            button.hide();
            softKeyboardVisible = false;
        }
    }

    return {
        setup: function (conn, btn) {
            if (bowser.mobile || bowser.tablet) {
                featurehandler.addFeatureListener("keyboard", enableSoftKeyboard);
                connection = conn;
                button = btn;
                softKeyboardVisible = false;
            } else {
                btn.hide();
            }
        }
    };
});
