"use strict";

define(function () {

    /*
    * `Factory` constructor.
    */
    function Factory () {
        this.mechs = [];
    }

    /*
    * Utilize the given `mech` with optional `name`, overridding the mechanism's
    * default name.
    *
    * Examples:
    *
    *     factory.use(FooMechanism);
    *
    *     factory.use('XFOO', FooMechanism);
    *
    * @param {String|Mechanism} name
    * @param {Mechanism} mech
    * @return {Factory} for chaining
    * @api public
    */
    Factory.prototype.use = function (name, mech) {
        var newMech = {};

        if (!mech) {
            newMech.Mech = name;
            newMech.name = newMech.Mech.prototype.name;
        } else {
            newMech.Mech = mech;
            newMech.name = name;
        }

        this.mechs.push(newMech);
        return this;
    };

    /*
    * Create a new mechanism from supported list of `mechs`.
    *
    * If no mechanisms are supported, returns `null`.
    *
    * Examples:
    *
    *     var mech = factory.create(['FOO', 'BAR']);
    *
    * @param {Array} mechs
    * @return {Mechanism}
    * @api public
    */
    Factory.prototype.create = function (mechs) {
        var i, j, entry, len, jlen;

        for (i = 0, len = this.mechs.length; i < len; i++) {
            for (j = 0, jlen = mechs.length; j < jlen; j++) {
                entry = this.mechs[i];
                if (entry.name === mechs[j]) {
                    return new entry.Mech();
                }
            }
        }

        return null;
    };

    return Factory;
});

/* ex:set ts=4 et: */

